//====== Copyright  1996-2005, Valve Corporation, All rights reserved. =======
//
// Purpose: 
//
//=============================================================================

#ifndef VEHICLE_JEEP_H
#define VEHICLE_JEEP_H
#ifdef _WIN32
#pragma once
#endif

#include "vehicle_base.h"
#include "ai_basenpc.h"
#include "hl2_vehicle_radar.h"

class CParticleSystem;
class CVehicleCargoTrigger;
class CSprite;

#define NUM_WHEEL_EFFECTS	2
#define NUM_HAZARD_LIGHTS	4
#define JEEP_WHEEL_COUNT	4

struct JeepWaterData_t
{
	bool		m_bWheelInWater[JEEP_WHEEL_COUNT];
	bool		m_bWheelWasInWater[JEEP_WHEEL_COUNT];
	Vector		m_vecWheelContactPoints[JEEP_WHEEL_COUNT];
	float		m_flNextRippleTime[JEEP_WHEEL_COUNT];
	bool		m_bBodyInWater;
	bool		m_bBodyWasInWater;

	DECLARE_SIMPLE_DATADESC();
};

//-----------------------------------------------------------------------------
// Purpose: 
//-----------------------------------------------------------------------------
class CPropJeep : public CPropVehicleDriveable
{
	DECLARE_CLASS( CPropJeep, CPropVehicleDriveable );

public:

	DECLARE_SERVERCLASS();
	DECLARE_DATADESC();

	CPropJeep( void );

	// CPropVehicle
	virtual void	ProcessMovement( CBasePlayer *pPlayer, CMoveData *pMoveData );
	virtual void	DriveVehicle( float flFrameTime, CUserCmd *ucmd, int iButtonsDown, int iButtonsReleased );
	virtual void	SetupMove( CBasePlayer *player, CUserCmd *ucmd, IMoveHelper *pHelper, CMoveData *move );
	virtual void	Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	virtual void	DampenEyePosition( Vector &vecVehicleEyePos, QAngle &vecVehicleEyeAngles );
	virtual bool	AllowBlockedExit( CBaseCombatCharacter *pPassenger, int nRole );
	virtual bool	CanExitVehicle( CBaseEntity *pEntity );
	virtual bool	IsVehicleBodyInWater() { return m_WaterData.m_bBodyInWater; }
	
	// Passengers always take damage except they are exiting.
	virtual bool PassengerShouldReceiveDamage( CTakeDamageInfo &info ) 
	{ 
		if ( GetServerVehicle() && GetServerVehicle()->IsPassengerExiting() )
			return false;

		//if ( info.GetDamageType() & DMG_VEHICLE )
		//	return true;

		//return (info.GetDamageType() & (DMG_RADIATION|DMG_BLAST) ) == 0; 
		return true;
	}

	virtual int		ObjectCaps( void ) { return (BaseClass::ObjectCaps() | FCAP_NOTIFY_ON_TRANSITION); }

	// CBaseEntity
	void			Think(void);
	void			Precache( void );
	void			Spawn( void ); 
	void			Activate( void );
	void			UpdateOnRemove( void );

	void			VehicleRecognition( void );

	virtual void	NPC_FinishedEnterVehicle( CAI_BaseNPC *pPassenger, bool bCompanion );
	virtual void	NPC_FinishedExitVehicle( CAI_BaseNPC *pPassenger, bool bCompanion );

	virtual bool	NPC_CanEnterVehicle( CAI_BaseNPC *pPassenger, bool bCompanion );
	virtual bool	NPC_CanExitVehicle( CAI_BaseNPC *pPassenger, bool bCompanion );

	virtual void	CreateServerVehicle( void );
	virtual Vector	BodyTarget( const Vector &posSrc, bool bNoisy = true );
	virtual void	TraceAttack( const CTakeDamageInfo &info, const Vector &vecDir, trace_t *ptr );
	virtual int		OnTakeDamage( const CTakeDamageInfo &info );
	virtual float	PassengerDamageModifier( const CTakeDamageInfo &info );

	virtual void	EnterVehicle( CBaseCombatCharacter *pPassenger );
	virtual void	ExitVehicle( int nRole );

	void			AimGunAt( Vector *endPos, float flInterval );
	bool			TauCannonHasBeenCutOff( void ) { return m_bGunHasBeenCutOff; }

	void			SpawnRadarPanel();
	void			DestroyRadarPanel();
	int				NumRadarContacts() { return m_iNumRadarContacts; }

	void			AddPropToCargoHold( CPhysicsProp *pProp );

	virtual CBaseEntity *OnFailedPhysGunPickup( Vector vPhysgunPos );
	virtual int		DrawDebugTextOverlays( void );

	// NPC Driving
	bool			NPC_HasPrimaryWeapon( void ) { return true; }
	void			NPC_AimPrimaryWeapon( Vector vecTarget );

	const char		*GetTracerType( void ) { return "AR2Tracer"; }
	void			DoImpactEffect( trace_t &tr, int nDamageType );

	bool			HeadlightIsOn( void ) { return m_bHeadlightIsOn; }
	void			HeadlightTurnOn( void );
	void			HeadlightTurnOff( void );

private:

	void		UpdateWheelDust( void );
	void		UpdateRadar( bool forceUpdate = false );

	void		InputLockEntrance( inputdata_t &data );
	void		InputUnlockEntrance( inputdata_t &data );
	void		InputLockExit( inputdata_t &data );
	void		InputUnlockExit( inputdata_t &data );
	void		InputEnableRadar( inputdata_t &data );
	void		InputDisableRadar( inputdata_t &data );
	void		InputEnableRadarDetectEnemies( inputdata_t &data );
	void		InputAddBusterToCargo( inputdata_t &data );
	void		InputSetCargoVisibility( inputdata_t &data );
	void		InputOutsideTransition( inputdata_t &data );
	void		InputDisablePhysGun( inputdata_t &data );
	void		InputEnablePhysGun( inputdata_t &data );
	void		InputCreateLinkController( inputdata_t &data );
	void		InputDestroyLinkController( inputdata_t &data );
	void		CreateAvoidanceZone( void );

	void		FireCannon( void );
	void		ChargeCannon( void );
	void		FireChargedCannon( void );

	void		DrawBeam( const Vector &startPos, const Vector &endPos, float width );
	void		StopChargeSound( void );
	void		GetCannonAim( Vector *resultDir );

	void		InitWaterData( void );
	void		CheckWaterLevel( void );
	void		CreateSplash( const Vector &vecPosition );
	void		CreateRipple( const Vector &vecPosition );

	void		CreateDangerSounds( void );

	void		ComputePDControllerCoefficients( float *pCoefficientsOut, float flFrequency, float flDampening, float flDeltaTime );
	void		DampenForwardMotion( Vector &vecVehicleEyePos, QAngle &vecVehicleEyeAngles, float flFrameTime );
	void		DampenUpMotion( Vector &vecVehicleEyePos, QAngle &vecVehicleEyeAngles, float flFrameTime );

	void		JeepSeagullThink( void );
	void		SpawnPerchedSeagull( void );
	void		AddSeagullPoop( const Vector &vecOrigin );

	void		InputShowHudHint( inputdata_t &inputdata );
	void		InputStartRemoveTauCannon( inputdata_t &inputdata );
	void		InputFinishRemoveTauCannon( inputdata_t &inputdata );

	bool	m_bEntranceLocked;
	bool	m_bExitLocked;
	bool	m_bAddingCargo;
	bool	m_bBlink;

	float	m_flCargoStartTime;	// Time when the cargo was first added to the vehicle (used for animating into hold)
	float	m_flNextAvoidBroadcastTime; // Next time we'll warn entity to move out of us

	COutputEvent	m_OnCompanionEnteredVehicle;	// Passenger has completed entering the vehicle
	COutputEvent	m_OnCompanionExitedVehicle;		// Passenger has completed exited the vehicle
	COutputEvent	m_OnHostileEnteredVehicle;		// Passenger has completed entering the vehicle
	COutputEvent	m_OnHostileExitedVehicle;		// Passenger has completed exited the vehicle

	CHandle< CParticleSystem >			m_hWheelDust[NUM_WHEEL_EFFECTS];
	CHandle< CParticleSystem >			m_hWheelWater[NUM_WHEEL_EFFECTS];
	CHandle< CVehicleCargoTrigger >		m_hCargoTrigger;
	CHandle< CPhysicsProp >				m_hCargoProp;

	CHandle< CSprite >					m_hHazardLights[NUM_HAZARD_LIGHTS];
	float								m_flNextWaterSound;

	bool	m_bRadarEnabled;
	bool	m_bRadarDetectsEnemies;
	float	m_flNextRadarUpdateTime;
	EHANDLE	m_hRadarScreen;

	EHANDLE	m_hLinkControllerFront;
	EHANDLE m_hLinkControllerRear;

	bool	m_bBusterHopperVisible;	// is the hopper assembly visible on the vehicle? please do not set this directly - use the accessor funct.

	bool	m_bIsBuggy;
	bool	m_bIsJalopy;
	bool	m_bIsRecognized;

	CNetworkVar( int, m_iNumRadarContacts );
	CNetworkArray( Vector, m_vecRadarContactPos, RADAR_MAX_CONTACTS );
	CNetworkArray( int, m_iRadarContactType, RADAR_MAX_CONTACTS );

protected:

	void			HazardBlinkThink( void );
	void			CreateHazardLights( void );
	void			DestroyHazardLights( void );

	void			UpdateCargoEntry( void );
	void			ReleasePropFromCargoHold( void );
	void			CreateCargoTrigger( void );
	virtual float	GetUprightTime( void ) { return 1.0f; }
	virtual float	GetUprightStrength( void );
	virtual bool	ShouldPuntUseLaunchForces( PhysGunForce_t reason ) { return ( reason == PHYSGUN_FORCE_PUNTED ); }
	virtual void	HandleWater( void );
	bool			CheckWater( void );

	virtual AngularImpulse	PhysGunLaunchAngularImpulse( void );
	virtual Vector			PhysGunLaunchVelocity( const Vector &forward, float flMass );
	bool					PassengerInTransition( void );

	void			SetBusterHopperVisibility(bool visible);


	bool			m_bGunHasBeenCutOff;
	float			m_flDangerSoundTime;
	int				m_nBulletType;
	bool			m_bCannonCharging;
	float			m_flCannonTime;
	float			m_flCannonChargeStartTime;
	Vector			m_vecGunOrigin;
	CSoundPatch		*m_sndCannonCharge;
	int				m_nSpinPos;
	float			m_aimYaw;
	float			m_aimPitch;
	float			m_throttleDisableTime;
	float			m_flAmmoCrateCloseTime;

	// handbrake after the fact to keep vehicles from rolling
	float			m_flHandbrakeTime;
	bool			m_bInitialHandbrake;

	float			m_flOverturnedTime;

	Vector			m_vecLastEyePos;
	Vector			m_vecLastEyeTarget;
	Vector			m_vecEyeSpeed;
	Vector			m_vecTargetSpeed;

	JeepWaterData_t	m_WaterData;

	int				m_iNumberOfEntries;
	int				m_nAmmoType;

	// Seagull perching
	float			m_flPlayerExitedTime;	// Time at which the player last left this vehicle
	float			m_flLastSawPlayerAt;	// Time at which we last saw the player
	EHANDLE			m_hLastPlayerInVehicle;
	EHANDLE			m_hSeagull;
	bool			m_bHasPoop;

	CNetworkVar( bool, m_bHeadlightIsOn );
};

#endif // VEHICLE_JEEP_H
